/************************************************************************/
/*	Author: 	Nate Eastland*/
/*	Copyright (c) 2017, Digilent Inc, All rights reserved		        */
/************************************************************************/
/*  Module Description: 												*/
/*																		*/
/*		This is for the MMC34166 magnetic sensor of the PmodCMPS2.							    */
/*																		*/
/************************************************************************/
/*  	Revision History:												*/
/*                                                                      */
/*
        1/4/2017    (NateE) created for component MMC3416xPJ*/
/*																		*/
/************************************************************************/

/************************************************************************/
/*                                                                      */
/*  This library is free software; you can redistribute it and/or       */
/*  modify it under the terms of the GNU Lesser General Public          */
/*  License as published by the Free Software Foundation; either        */
/*  version 2.1 of the License, or (at your option) any later version.  */
/*                                                                      */
/*  This library is distributed in the hope that it will be useful,     */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  Lesser General Public License for more details.                     */
/*                                                                      */
/*  You should have received a copy of the GNU Lesser General Public    */
/*  License along with this library; if not, write to the Free Software */
/*  Foundation, Inc.,51 Franklin St, Fifth Floor, Boston, MA  02110-1301*/
/*  USA                                                                 */
/************************************************************************/

/**********************************************
Function:

Date Last Modified: *
Description:
Input Param:
Return:
**********************************************/

#include <CMPS2.h>

/**********************************************
Function: CMPS2()

Date Last Modified: *
Description: default constructor
Input Param: none
Return: N/A
**********************************************/

CMPS2::CMPS2()
{
  this->xLow = 0;
  this->xHigh = 0;
  this->yLow = 0;
  this->yHigh = 0;
  this->zLow = 0;
  this->zHigh = 0;
  this->offset = 0;
  this->declination = 0;

  this->xMin = 32766;
  this->yMin = 32766;
  this->zMin = 32766;

  this->xMax = 32768;
  this->yMax = 32768;
  this->zMax = 32768;

  this->xAdj = 0;
  this->yAdj = 0;
  this->zAdj = 0;

  this->deviceID = 0x60;

  uint8_t* i2cHandlePtr = (uint8_t*)(&this->i2cHandle);
  for(uint32_t i = 0; i < sizeof(I2C_HandleTypeDef); i++)
    i2cHandlePtr[i] = 0;
}


uint8_t CMPS2::readReg(uint8_t addr)
{
  uint8_t data;
  HAL_I2C_Mem_Read(&this->i2cHandle, this->deviceID, addr, 1, &data, 1, 100);
  return data;
}

void CMPS2::writeReg(uint8_t addr, uint8_t data)
{
  HAL_I2C_Mem_Write(&this->i2cHandle, this->deviceID, addr, 1, &data, 1, 100);
}

/**********************************************
Function: init()

Date Last Modified: *
Description: Initialization function for the device. Performes a SET operation
    as a means of preparing the device for measuremtns.
Input Param: none
Return: void
**********************************************/

void CMPS2::init()
{
  __HAL_RCC_I2C4_CLK_ENABLE();
  __HAL_RCC_GPIOF_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF4_I2C4;
  GPIO_InitStruct.Pin = GPIO_PIN_14 | GPIO_PIN_15;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

  this->i2cHandle.Instance = I2C4;
  this->i2cHandle.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  this->i2cHandle.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  this->i2cHandle.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  this->i2cHandle.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  this->i2cHandle.Init.OwnAddress1 = 0x01;

  // Set the I2C_TIMINGR register to fulfill the MMC3416xPJ timing requirements.
  this->i2cHandle.Init.Timing = 0x10563046;

  HAL_I2C_Init(&this->i2cHandle);

  this->writeReg(intConReg0, 0x20);

  HAL_Delay(10);

  for(int i = 0;i < 100000; i++); //delay to allow the manufacturer suggested 10ms elapsed time for SET to execute

  this->writeReg(intConReg1, 0x00);
}

/**********************************************
Function: getMeasurement()

Date Last Modified: *
Description: This function performes a single measuremnt operation for the X, Y, and Z coordinates
    measured by the device. Each coordinate is sent as an upper and lower byte. Following a single
    measurement operation the device enters a low-power wait state per the datasheet.
Input Param: none
Return: Returned integer indicates failure of communication with device based on return values
  of endTransmission(). Return value of 0 indicates successful communication.
**********************************************/
int CMPS2::getMeasurement()
{
  int fail = 0;
  int temp = 0;
  //uint8_t xL, xH, yL, yH, zL, zH;

  this->writeReg(intConReg0, 0x01);

  do{
    temp = this->readReg(statusReg) & 0x01;
  } while(temp != readReady);

  this->xLow = this->readReg(0x00);
  this->xHigh = this->readReg(0x01);

  this->yLow = this->readReg(0x02);
  this->yHigh = this->readReg(0x03);

  this->zLow = this->readReg(0x04);
  this->zHigh = this->readReg(0x05);

  /*
    The following section determines new max, min, and mid values for x,y, and z
    for heading calibration.
  */

  this->x = getX();
  this->y = getY();
  this->z = getZ();

  if(this->x > this->xMax)
    this->xMax = this->x;
  if(this->x < this->xMin)
    this->xMin = this->x;

  if(this->y > this->yMax)
    this->yMax = this->y;
  if(this->y < this->yMin)
    this->yMin = this->y;

  if(this->z > this->zMax)
    this->zMax = this->z;
  if(this->z < this->zMin)
    this->zMin = this->z;


  this->xMid = (this->xMax + this->xMin) / 2;
  this->yMid = (this->yMax + this->yMin) / 2;
  this->zMid = (this->zMax + this->zMin) / 2;

  return fail;
}

/**********************************************
Function: degree()

Date Last Modified: *
Description: This funciton determines the degree reading of the sensor.
    The reading is adjusted to match with conventional compass readings. This does
    not currently include any elliptical calculation to compensate for tilt of the
    sensor, so these readings will only be accurate when device is operated on a level
    surface
Input Param: none
Return: degree measurement integer
**********************************************/
int CMPS2::degree()
{
  int temp0 = 0;
  int temp1 = 0;
  int deg = 0;

  if(this->x < this->xMid)
  {
    if(this->y > this->yMid)
    {
      //Quadrant 1
      temp0 = this->y - this->yMid;
      temp1 = this->xMid - this->x;

      deg = (90) - ( atan((double)((temp0)/(double)(temp1))) * (180/3.14159) );
    }
    else
    {
      //Quadrant 2
      temp0 = this->yMid - this->y;
      temp1 = this->xMid - this->x;

      deg = (90) + ( atan((double)((temp0)/(double)(temp1))) * (180/3.14159) );
    }
  }
  else
  {
    if(this->y < this->yMid)
    {
      //Quadrant 3
      temp0 = this->yMid - this->y;
      temp1 = this->x - this->xMid;

      deg = (270) - ( atan((double)((temp0)/(double)(temp1))) * (180/3.14159) );
    }
    else
    {
      //Quadrant 4
      temp0 = this->y - this->yMid;
      temp1 = this->x - this->xMid;

      deg = (270) + ( atan((double)((temp0)/(double)(temp1))) * (180/3.14159) );
    }
  }

  deg = deg + declination;

  if(declination > 0)
  {
      if(deg > 360)
        deg = deg - 360;
  }
  else
  {
      if(deg < 0)
        deg = 360 + deg;
  }

  return deg;
}

/**********************************************
Function: readHeading()

Date Last Modified: *
Description: directs the device to take a measurement then subsequently uses
    that measurement data to determine a compass degree heading.
Input Param: none
Return: degree measurement integer
**********************************************/
int CMPS2::readHeading()
{
  int degOut = 0;

  getMeasurement();

  degOut = degree();

  return degOut;
}

/**********************************************
Function: getX()

Date Last Modified: *
Description: This function combines the upper and lower byte of the X measurement
    into a single integer.
Input Param: none
Return: integer measurement of x
**********************************************/
int CMPS2::getX()
{
  int temp = 0;

  temp = ((this->xHigh) << 8) + this->xLow;

  return temp;
}

/**********************************************
Function: getY()

Date Last Modified: *
Description:This function combines the upper and lower byte of the Y measurement
    into a single integer.
Input Param: none
Return: integer measurement of Y
**********************************************/
int CMPS2::getY()
{
  int temp = 0;

  temp = ((this->yHigh) << 8) + this->yLow;

  return temp;
}

/**********************************************
Function: getZ()

Date Last Modified: *
Description: This function combines the upper and lower byte of the Z measurement
    into a single integer.
Input Param: none
Return: integer measurement of Z
**********************************************/
int CMPS2::getZ()
{
  int temp = 0;

  temp = ((this->zHigh) << 8) + this->zLow;

  return temp;
}

/**********************************************
Function: getOffset()

Date Last Modified: *
Description: getter for thermal drift offset value
Input Param:
Return:
**********************************************/
double CMPS2::getOffset()
{
  return this->offset;
}

/**********************************************
Function: setOffset()

Date Last Modified: *
Description: setter for thermal drift offset value
Input Param:
Return:
**********************************************/
void CMPS2::setOffset(double setVal)
{
  this->offset = setVal;
}

int CMPS2::getT1()
{
    return this->test1;
}

int CMPS2::getT2()
{
  return this->test2;
}

/**********************************************
Function: setDevID()

Date Last Modified: *
Description: The purpose of this funciton is to allow for setting a device ID
    for a new CMPS object for projects in which multiple sensors are used (up to 8).
Input Param: newID as a uint8_t
Return: nothing
**********************************************/
void CMPS2::setDevID(uint8_t newID)
{
  this->deviceID = newID;
}

/**********************************************
Function: setDeclination()

Date Last Modified: *
Description: this function allows the user to set a degree of magnetic declination
    to be used as compenstion in heading calculation.
Input Param: newDec, can be entered as a signed integer
Return: nothing
**********************************************/
void CMPS2::setDeclination(int newDec)
{
  this->declination = newDec;
}
